import functools

try:
    cache = functools.cache
except AttributeError:
    cache = functools.lru_cache


class SyscallTableArm(object):
    def __init__(self, bits: int):
        self.bits = bits

    @cache
    def __getitem__(self, name: str) -> int:
        if self.bits == 32:
            if name in _syscall_table_32:
                return _syscall_table_32[name]

        else:
            if name in _syscall_table_64:
                return _syscall_table_64[name]

        raise KeyError("No such system call: {}".format(name))

    def __getattr__(self, name: str) -> int:
        return self[name]

_syscall_table_32 = {'restart_syscall': 0, 'exit': 1, 'fork': 2, 'read': 3, 'write': 4, 'open': 5, 'close': 6, 'waitpid': 7, 'creat': 8, 'link': 9, 'unlink': 10, 'execve': 11, 'chdir': 12, 'time32': 13, 'mknod': 14, 'chmod': 15, 'lchown16': 16, 'break': 17, 'stat': 18, 'lseek': 19, 'getpid': 20, 'mount': 21, 'oldumount': 22, 'setuid16': 23, 'getuid16': 24, 'stime32': 25, 'ptrace': 26, 'alarm': 27, 'fstat': 28, 'pause': 29, 'utime32': 30, 'stty': 31, 'gtty': 32, 'access': 33, 'nice': 34, 'ftime': 35, 'sync': 36, 'kill': 37, 'rename': 38, 'mkdir': 39, 'rmdir': 40, 'dup': 41, 'pipe': 42, 'times': 43, 'prof': 44, 'brk': 45, 'setgid16': 46, 'getgid16': 47, 'signal': 48, 'geteuid16': 49, 'getegid16': 50, 'acct': 51, 'umount': 52, 'lock': 53, 'ioctl': 54, 'fcntl': 55, 'mpx': 56, 'setpgid': 57, 'ulimit': 58, 'olduname': 59, 'umask': 60, 'chroot': 61, 'ustat': 62, 'dup2': 63, 'getppid': 64, 'getpgrp': 65, 'setsid': 66, 'sigaction': 67, 'sgetmask': 68, 'ssetmask': 69, 'setreuid16': 70, 'setregid16': 71, 'sigsuspend': 72, 'sigpending': 73, 'sethostname': 74, 'setrlimit': 75, 'old_getrlimit': 76, 'getrusage': 77, 'gettimeofday': 78, 'settimeofday': 79, 'getgroups16': 80, 'setgroups16': 81, 'old_select': 82, 'symlink': 83, 'lstat': 84, 'readlink': 85, 'uselib': 86, 'swapon': 87, 'reboot': 88, 'old_readdir': 89, 'old_mmap': 90, 'munmap': 91, 'truncate': 92, 'ftruncate': 93, 'fchmod': 94, 'fchown16': 95, 'getpriority': 96, 'setpriority': 97, 'profil': 98, 'statfs': 99, 'fstatfs': 100, 'ioperm': 101, 'oabi_socketcall': 102, 'syslog': 103, 'setitimer': 104, 'getitimer': 105, 'newstat': 106, 'newlstat': 107, 'newfstat': 108, 'uname': 109, 'iopl': 110, 'vhangup': 111, 'idle': 112, 'syscall': 113, 'wait4': 114, 'swapoff': 115, 'sysinfo': 116, 'oabi_ipc': 117, 'fsync': 118, 'sigreturn_wrapper': 119, 'clone': 120, 'setdomainname': 121, 'newuname': 122, 'modify_ldt': 123, 'adjtimex_time32': 124, 'mprotect': 125, 'sigprocmask': 126, 'create_module': 127, 'init_module': 128, 'delete_module': 129, 'get_kernel_syms': 130, 'quotactl': 131, 'getpgid': 132, 'fchdir': 133, 'ni_syscall': 149, 'sysfs': 135, 'personality': 136, 'afs_syscall': 137, 'setfsuid16': 138, 'setfsgid16': 139, 'llseek': 140, 'getdents': 141, 'select': 142, 'flock': 143, 'msync': 144, 'readv': 145, 'writev': 146, 'getsid': 147, 'fdatasync': 148, 'mlock': 150, 'munlock': 151, 'mlockall': 152, 'munlockall': 153, 'sched_setparam': 154, 'sched_getparam': 155, 'sched_setscheduler': 156, 'sched_getscheduler': 157, 'sched_yield': 158, 'sched_get_priority_max': 159, 'sched_get_priority_min': 160, 'sched_rr_get_interval_time32': 161, 'nanosleep_time32': 162, 'mremap': 163, 'setresuid16': 164, 'getresuid16': 165, 'vm86': 166, 'query_module': 167, 'poll': 168, 'setresgid16': 170, 'getresgid16': 171, 'prctl': 172, 'rt_sigreturn_wrapper': 173, 'rt_sigaction': 174, 'rt_sigprocmask': 175, 'rt_sigpending': 176, 'rt_sigtimedwait_time32': 177, 'rt_sigqueueinfo': 178, 'rt_sigsuspend': 179, 'oabi_pread64': 180, 'oabi_pwrite64': 181, 'chown16': 182, 'getcwd': 183, 'capget': 184, 'capset': 185, 'sigaltstack': 186, 'sendfile': 187, 'vfork': 190, 'getrlimit': 191, 'mmap2': 192, 'oabi_truncate64': 193, 'oabi_ftruncate64': 194, 'oabi_stat64': 195, 'oabi_lstat64': 196, 'oabi_fstat64': 197, 'lchown': 198, 'getuid': 199, 'getgid': 200, 'geteuid': 201, 'getegid': 202, 'setreuid': 203, 'setregid': 204, 'getgroups': 205, 'setgroups': 206, 'fchown': 207, 'setresuid': 208, 'getresuid': 209, 'setresgid': 210, 'getresgid': 211, 'chown': 212, 'setuid': 213, 'setgid': 214, 'setfsuid': 215, 'setfsgid': 216, 'getdents64': 217, 'pivot_root': 218, 'mincore': 219, 'madvise': 220, 'oabi_fcntl64': 221, 'gettid': 224, 'oabi_readahead': 225, 'setxattr': 226, 'lsetxattr': 227, 'fsetxattr': 228, 'getxattr': 229, 'lgetxattr': 230, 'fgetxattr': 231, 'listxattr': 232, 'llistxattr': 233, 'flistxattr': 234, 'removexattr': 235, 'lremovexattr': 236, 'fremovexattr': 237, 'tkill': 238, 'sendfile64': 239, 'futex_time32': 240, 'sched_setaffinity': 241, 'sched_getaffinity': 242, 'io_setup': 243, 'io_destroy': 244, 'io_getevents_time32': 245, 'io_submit': 246, 'io_cancel': 247, 'exit_group': 248, 'lookup_dcookie': 249, 'epoll_create': 250, 'oabi_epoll_ctl': 251, 'epoll_wait': 252, 'remap_file_pages': 253, 'set_tid_address': 256, 'timer_create': 257, 'timer_settime32': 258, 'timer_gettime32': 259, 'timer_getoverrun': 260, 'timer_delete': 261, 'clock_settime32': 262, 'clock_gettime32': 263, 'clock_getres_time32': 264, 'clock_nanosleep_time32': 265, 'statfs64_wrapper': 266, 'fstatfs64_wrapper': 267, 'tgkill': 268, 'utimes_time32': 269, 'arm_fadvise64_64': 270, 'pciconfig_iobase': 271, 'pciconfig_read': 272, 'pciconfig_write': 273, 'mq_open': 274, 'mq_unlink': 275, 'mq_timedsend_time32': 276, 'mq_timedreceive_time32': 277, 'mq_notify': 278, 'mq_getsetattr': 279, 'waitid': 280, 'socket': 281, 'oabi_bind': 282, 'oabi_connect': 283, 'listen': 284, 'accept': 285, 'getsockname': 286, 'getpeername': 287, 'socketpair': 288, 'send': 289, 'oabi_sendto': 290, 'recv': 291, 'recvfrom': 292, 'shutdown': 293, 'setsockopt': 294, 'getsockopt': 295, 'oabi_sendmsg': 296, 'recvmsg': 297, 'oabi_semop': 298, 'semget': 299, 'old_semctl': 300, 'msgsnd': 301, 'msgrcv': 302, 'msgget': 303, 'old_msgctl': 304, 'shmat': 305, 'shmdt': 306, 'shmget': 307, 'old_shmctl': 308, 'add_key': 309, 'request_key': 310, 'keyctl': 311, 'oabi_semtimedop': 312, 'ioprio_set': 314, 'ioprio_get': 315, 'inotify_init': 316, 'inotify_add_watch': 317, 'inotify_rm_watch': 318, 'mbind': 319, 'get_mempolicy': 320, 'set_mempolicy': 321, 'openat': 322, 'mkdirat': 323, 'mknodat': 324, 'fchownat': 325, 'futimesat_time32': 326, 'oabi_fstatat64': 327, 'unlinkat': 328, 'renameat': 329, 'linkat': 330, 'symlinkat': 331, 'readlinkat': 332, 'fchmodat': 333, 'faccessat': 334, 'pselect6_time32': 335, 'ppoll_time32': 336, 'unshare': 337, 'set_robust_list': 338, 'get_robust_list': 339, 'splice': 340, 'sync_file_range2': 341, 'tee': 342, 'vmsplice': 343, 'move_pages': 344, 'getcpu': 345, 'epoll_pwait': 346, 'kexec_load': 347, 'utimensat_time32': 348, 'signalfd': 349, 'timerfd_create': 350, 'eventfd': 351, 'fallocate': 352, 'timerfd_settime32': 353, 'timerfd_gettime32': 354, 'signalfd4': 355, 'eventfd2': 356, 'epoll_create1': 357, 'dup3': 358, 'pipe2': 359, 'inotify_init1': 360, 'preadv': 361, 'pwritev': 362, 'rt_tgsigqueueinfo': 363, 'perf_event_open': 364, 'recvmmsg_time32': 365, 'accept4': 366, 'fanotify_init': 367, 'fanotify_mark': 368, 'prlimit64': 369, 'name_to_handle_at': 370, 'open_by_handle_at': 371, 'clock_adjtime32': 372, 'syncfs': 373, 'sendmmsg': 374, 'setns': 375, 'process_vm_readv': 376, 'process_vm_writev': 377, 'kcmp': 378, 'finit_module': 379, 'sched_setattr': 380, 'sched_getattr': 381, 'renameat2': 382, 'seccomp': 383, 'getrandom': 384, 'memfd_create': 385, 'bpf': 386, 'execveat': 387, 'userfaultfd': 388, 'membarrier': 389, 'mlock2': 390, 'copy_file_range': 391, 'preadv2': 392, 'pwritev2': 393, 'pkey_mprotect': 394, 'pkey_alloc': 395, 'pkey_free': 396, 'statx': 397, 'rseq': 398, 'io_pgetevents_time32': 399, 'migrate_pages': 400, 'kexec_file_load': 401, 'clock_gettime': 403, 'clock_settime': 404, 'clock_adjtime': 405, 'clock_getres': 406, 'clock_nanosleep': 407, 'timer_gettime': 408, 'timer_settime': 409, 'timerfd_gettime': 410, 'timerfd_settime': 411, 'utimensat': 412, 'pselect6': 413, 'ppoll': 414, 'io_pgetevents': 416, 'recvmmsg': 417, 'mq_timedsend': 418, 'mq_timedreceive': 419, 'semtimedop': 420, 'rt_sigtimedwait': 421, 'futex': 422, 'sched_rr_get_interval': 423, 'pidfd_send_signal': 424, 'io_uring_setup': 425, 'io_uring_enter': 426, 'io_uring_register': 427, 'open_tree': 428, 'move_mount': 429, 'fsopen': 430, 'fsconfig': 431, 'fsmount': 432, 'fspick': 433, 'pidfd_open': 434, 'clone3': 435, 'close_range': 436, 'openat2': 437, 'pidfd_getfd': 438, 'faccessat2': 439, 'process_madvise': 440, 'epoll_pwait2': 441, 'mount_setattr': 442, 'quotactl_fd': 443, 'landlock_create_ruleset': 444, 'landlock_add_rule': 445, 'landlock_restrict_self': 446, 'process_mrelease': 448, 'futex_waitv': 449, 'set_mempolicy_home_node': 450}

_syscall_table_64 = {'io_setup': 0, 'io_destroy': 1, 'io_submit': 2, 'io_cancel': 3, 'io_getevents': 4, 'setxattr': 5, 'lsetxattr': 6, 'fsetxattr': 7, 'getxattr': 8, 'lgetxattr': 9, 'fgetxattr': 10, 'listxattr': 11, 'llistxattr': 12, 'flistxattr': 13, 'removexattr': 14, 'lremovexattr': 15, 'fremovexattr': 16, 'getcwd': 17, 'lookup_dcookie': 18, 'eventfd2': 19, 'epoll_create1': 20, 'epoll_ctl': 21, 'epoll_pwait': 22, 'dup': 23, 'dup3': 24, 'inotify_init1': 26, 'inotify_add_watch': 27, 'inotify_rm_watch': 28, 'ioctl': 29, 'ioprio_set': 30, 'ioprio_get': 31, 'flock': 32, 'mknodat': 33, 'mkdirat': 34, 'unlinkat': 35, 'symlinkat': 36, 'linkat': 37, 'renameat': 38, 'umount2': 39, 'mount': 40, 'pivot_root': 41, 'nfsservctl': 42, 'fallocate': 47, 'faccessat': 48, 'chdir': 49, 'fchdir': 50, 'chroot': 51, 'fchmod': 52, 'fchmodat': 53, 'fchownat': 54, 'fchown': 55, 'openat': 56, 'close': 57, 'vhangup': 58, 'pipe2': 59, 'quotactl': 60, 'getdents64': 61, 'read': 63, 'write': 64, 'readv': 65, 'writev': 66, 'pread64': 67, 'pwrite64': 68, 'preadv': 69, 'pwritev': 70, 'pselect6': 72, 'ppoll': 73, 'signalfd4': 74, 'vmsplice': 75, 'splice': 76, 'tee': 77, 'readlinkat': 78, 'sync': 81, 'fsync': 82, 'fdatasync': 83, 'sync_file_range2': 84, 'sync_file_range': 84, 'timerfd_create': 85, 'timerfd_settime': 86, 'timerfd_gettime': 87, 'utimensat': 88, 'acct': 89, 'capget': 90, 'capset': 91, 'personality': 92, 'exit': 93, 'exit_group': 94, 'waitid': 95, 'set_tid_address': 96, 'unshare': 97, 'futex': 98, 'set_robust_list': 99, 'get_robust_list': 100, 'nanosleep': 101, 'getitimer': 102, 'setitimer': 103, 'kexec_load': 104, 'init_module': 105, 'delete_module': 106, 'timer_create': 107, 'timer_gettime': 108, 'timer_getoverrun': 109, 'timer_settime': 110, 'timer_delete': 111, 'clock_settime': 112, 'clock_gettime': 113, 'clock_getres': 114, 'clock_nanosleep': 115, 'syslog': 116, 'ptrace': 117, 'sched_setparam': 118, 'sched_setscheduler': 119, 'sched_getscheduler': 120, 'sched_getparam': 121, 'sched_setaffinity': 122, 'sched_getaffinity': 123, 'sched_yield': 124, 'sched_get_priority_max': 125, 'sched_get_priority_min': 126, 'sched_rr_get_interval': 127, 'restart_syscall': 128, 'kill': 129, 'tkill': 130, 'tgkill': 131, 'sigaltstack': 132, 'rt_sigsuspend': 133, 'rt_sigaction': 134, 'rt_sigprocmask': 135, 'rt_sigpending': 136, 'rt_sigtimedwait': 137, 'rt_sigqueueinfo': 138, 'rt_sigreturn': 139, 'setpriority': 140, 'getpriority': 141, 'reboot': 142, 'setregid': 143, 'setgid': 144, 'setreuid': 145, 'setuid': 146, 'setresuid': 147, 'getresuid': 148, 'setresgid': 149, 'getresgid': 150, 'setfsuid': 151, 'setfsgid': 152, 'times': 153, 'setpgid': 154, 'getpgid': 155, 'getsid': 156, 'setsid': 157, 'getgroups': 158, 'setgroups': 159, 'uname': 160, 'sethostname': 161, 'setdomainname': 162, 'getrlimit': 163, 'setrlimit': 164, 'getrusage': 165, 'umask': 166, 'prctl': 167, 'getcpu': 168, 'gettimeofday': 169, 'settimeofday': 170, 'adjtimex': 171, 'getpid': 172, 'getppid': 173, 'getuid': 174, 'geteuid': 175, 'getgid': 176, 'getegid': 177, 'gettid': 178, 'sysinfo': 179, 'mq_open': 180, 'mq_unlink': 181, 'mq_timedsend': 182, 'mq_timedreceive': 183, 'mq_notify': 184, 'mq_getsetattr': 185, 'msgget': 186, 'msgctl': 187, 'msgrcv': 188, 'msgsnd': 189, 'semget': 190, 'semctl': 191, 'semtimedop': 192, 'semop': 193, 'shmget': 194, 'shmctl': 195, 'shmat': 196, 'shmdt': 197, 'socket': 198, 'socketpair': 199, 'bind': 200, 'listen': 201, 'accept': 202, 'connect': 203, 'getsockname': 204, 'getpeername': 205, 'sendto': 206, 'recvfrom': 207, 'setsockopt': 208, 'getsockopt': 209, 'shutdown': 210, 'sendmsg': 211, 'recvmsg': 212, 'readahead': 213, 'brk': 214, 'munmap': 215, 'mremap': 216, 'add_key': 217, 'request_key': 218, 'keyctl': 219, 'clone': 220, 'execve': 221, 'swapon': 224, 'swapoff': 225, 'mprotect': 226, 'msync': 227, 'mlock': 228, 'munlock': 229, 'mlockall': 230, 'munlockall': 231, 'mincore': 232, 'madvise': 233, 'remap_file_pages': 234, 'mbind': 235, 'get_mempolicy': 236, 'set_mempolicy': 237, 'migrate_pages': 238, 'move_pages': 239, 'rt_tgsigqueueinfo': 240, 'perf_event_open': 241, 'accept4': 242, 'recvmmsg': 243, 'arch_specific_syscall': 244, 'wait4': 260, 'prlimit64': 261, 'fanotify_init': 262, 'fanotify_mark': 263, 'name_to_handle_at': 264, 'open_by_handle_at': 265, 'clock_adjtime': 266, 'syncfs': 267, 'setns': 268, 'sendmmsg': 269, 'process_vm_readv': 270, 'process_vm_writev': 271, 'kcmp': 272, 'finit_module': 273, 'sched_setattr': 274, 'sched_getattr': 275, 'renameat2': 276, 'seccomp': 277, 'getrandom': 278, 'memfd_create': 279, 'bpf': 280, 'execveat': 281, 'userfaultfd': 282, 'membarrier': 283, 'mlock2': 284, 'copy_file_range': 285, 'preadv2': 286, 'pwritev2': 287, 'pkey_mprotect': 288, 'pkey_alloc': 289, 'pkey_free': 290, 'statx': 291, 'io_pgetevents': 292, 'rseq': 293, 'kexec_file_load': 294, 'clock_gettime64': 403, 'clock_settime64': 404, 'clock_adjtime64': 405, 'clock_getres_time64': 406, 'clock_nanosleep_time64': 407, 'timer_gettime64': 408, 'timer_settime64': 409, 'timerfd_gettime64': 410, 'timerfd_settime64': 411, 'utimensat_time64': 412, 'pselect6_time64': 413, 'ppoll_time64': 414, 'io_pgetevents_time64': 416, 'recvmmsg_time64': 417, 'mq_timedsend_time64': 418, 'mq_timedreceive_time64': 419, 'semtimedop_time64': 420, 'rt_sigtimedwait_time64': 421, 'futex_time64': 422, 'sched_rr_get_interval_time64': 423, 'pidfd_send_signal': 424, 'io_uring_setup': 425, 'io_uring_enter': 426, 'io_uring_register': 427, 'open_tree': 428, 'move_mount': 429, 'fsopen': 430, 'fsconfig': 431, 'fsmount': 432, 'fspick': 433, 'pidfd_open': 434, 'clone3': 435, 'close_range': 436, 'openat2': 437, 'pidfd_getfd': 438, 'faccessat2': 439, 'process_madvise': 440, 'epoll_pwait2': 441, 'mount_setattr': 442, 'quotactl_fd': 443, 'landlock_create_ruleset': 444, 'landlock_add_rule': 445, 'landlock_restrict_self': 446, 'memfd_secret': 447, 'process_mrelease': 448, 'futex_waitv': 449, 'set_mempolicy_home_node': 450, 'syscalls': 451}
